@@ -27,23 +27,31 @@ import signal
 import subprocess
 import traceback
 import time
+import easycluster as _core
+
+if _core.PYTHON3:
+    NAME = 'EasyCluster-Py3'
+    SUFFIX = '-py3'
+else:
+    NAME = 'EasyCluster'
+    SUFFIX = ''
 
-NAME = 'EasyCluster'
 MODULE = 'easycluster'
 MAIN_MODULE = MODULE + '.__main__'
-PIDFILE = '/var/run/easycluster.pid'
 
-KEY_PATH = '/etc/easycluster_service.key'
-INIT_PATH = '/etc/init.d/easycluster'
+PIDFILE = '/var/run/easycluster%s.pid' % SUFFIX
+KEY_PATH = '/etc/easycluster_service%s.key' % SUFFIX
+INIT_PATH = '/etc/init.d/easycluster%s' % SUFFIX
+SERVICE_NAME = 'easycluster%s' % SUFFIX
 
 INIT_SCRIPT = '''\
 #!/bin/sh
 # chkconfig: 345 99 01
 # description: EasyCluster remote service
-# processname: easycluster
+# processname: easycluster%(suffix)s
 
 ### BEGIN INIT INFO
-# Provides:          easycluster
+# Provides:          easycluster%(suffix)s
 # Required-Start:    $remote_fs $syslog
 # Required-Stop:     $remote_fs $syslog
 # Default-Start:     2 3 4 5
@@ -55,21 +63,21 @@ INIT_SCRIPT = '''\
 
 PATH=/sbin:/usr/sbin:/bin:/usr/bin
 
-exec '%s' -m %s "$@"
+exec '%(exe)s' -m %(name)s "$@"
 '''
 
 def install_service():
     temp_path = INIT_PATH + '~'
 
     with open(temp_path, 'w') as fp:
-        fp.write(INIT_SCRIPT %  (sys.executable, __name__))
+        fp.write(INIT_SCRIPT %  dict(exe=sys.executable, name=__name__, suffix=SUFFIX))
 
     os.chmod(temp_path, 0o755)
     os.rename(temp_path, INIT_PATH)
 
-    for prog in (['update-rc.d', MODULE, 'defaults'],
-                 ['chkconfig', '--add', MODULE],
-                 ['/usr/lib/lsb/install_initd', MODULE]):
+    for prog in (['update-rc.d', SERVICE_NAME, 'defaults'],
+                 ['chkconfig', '--add', SERVICE_NAME],
+                 ['/usr/lib/lsb/install_initd', SERVICE_NAME]):
         
         try:
             subprocess.call(prog)
@@ -81,9 +89,9 @@ def install_service():
         raise OSError("No method of installing initscripts found (tried Debian, Redhat, LSB)")
 
 def uninstall_service():
-    for prog in (['update-rc.d', '-f', MODULE, 'remove'],
-                 ['chkconfig', '--del', MODULE],
-                 ['/usr/lib/lsb/remove_initd', MODULE]):
+    for prog in (['update-rc.d', '-f', SERVICE_NAME, 'remove'],
+                 ['chkconfig', '--del', SERVICE_NAME],
+                 ['/usr/lib/lsb/remove_initd', SERVICE_NAME]):
         try:
             subprocess.call(prog)
         except OSError:
@@ -120,18 +128,20 @@ def is_daemon(pid, name):
         if not os.path.exists(cmdf):
             ## Proc not mounted or not Linux - just assume pidfile is valid
             return True
-
-        cmdline = file(cmdf, 'r').read().split('\0')
+        
+        with open(cmdf, 'r') as fp:
+            cmdline = fp.read().split('\0')
         if cmdline[1] == '-m' and cmdline[2] == name:
             return True
-    except Exception:
+    except (EnvironmentError, ValueError):
         pass
     return False
 
 def read_pidfile(fil):
     try:
-        return int(file(fil, 'r').read().strip())
-    except Exception:
+        with open(fil, 'r') as fp:
+            return int(fp.read().strip())
+    except (EnvironmentError, ValueError):
         return 0
 
 def getpid():
