@@ -496,6 +496,7 @@ def server_main(args=None):
     options.add_option('-O', '--overwrite', action='store_true', help='Allow --generate to overwrite existing keyfile')
     options.add_option('-p', '--port', type='int', default=_core.DEFAULT_PORT, help='TCP port to listen on. Default: %d' % _core.DEFAULT_PORT)
     options.add_option('-b', '--bind', default=None, help='Bind the server to a specific IP address')
+    options.add_option('-N', '--no-ipv6', action='store_true', help='Disable IPv6 support')
     options.add_option('-a', '--allow', action='append', default=[], help='Allow only a specific address or subnet to connect '
                        '(e.g. 192.168.0.0/24). Specify more than once to allow multiple addresses.')
 
@@ -574,12 +575,44 @@ def server_main(args=None):
 
         run_server(opts.port, key, opts.bind, opts.allow, opts.verbose,
                    getattr(opts, 'daemonize', False),
-                   getattr(opts, 'pidfile', None))
+                   getattr(opts, 'pidfile', None),
+                   opts.no_ipv6)
     else:
         options.print_help()
 
+def create_socket(family, bindaddr, port):
+
+    sockaddr = ('', port)
+    if bindaddr is not None:
+        gai_family = 0 if family == socket.AF_INET6 else family
+        addrs = socket.getaddrinfo(bindaddr, port, gai_family)
+        family, socktype, proto, canonname, sockaddr = addrs[0]
+
+    try:
+        sock = socket.socket(family, socket.SOCK_STREAM)
+        if family == socket.AF_INET6 and socket.has_ipv6:
+            # Windows needs this socket option set to listen on both IPV4 and
+            # IPV6. However, IPPROTO_IPV6 is missing from the socket module on Windows.
+            # Since the value is likely to never change, just include it here.
+            IPPROTO_IPV6 = 41
+            sock.setsockopt(IPPROTO_IPV6, socket.IPV6_V6ONLY, 0)
+
+        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+
+        _core._mark_non_inheritable(sock.fileno())
+
+        sock.bind(sockaddr)
+        return sock
+
+    except socket.error as e:
+        print(family, sockaddr)
+        # Fall back to IPv4
+        if family == socket.AF_INET6 and e.args[0] in (97, 10047):
+            return create_socket(socket.AF_INET, bindaddr, port)
+        raise
+
 def run_server(port, key, bindaddr=None, allow=(), 
-               verbosity=0, daemonize=False, pidfile=None):
+               verbosity=0, daemonize=False, pidfile=None, disable_ipv6=False):
     '''Runs a server on the specified port.
 
     @type  port: int
@@ -613,40 +646,13 @@ def run_server(port, key, bindaddr=None, allow=(),
 
     server_init()
 
-    if socket.has_ipv6:
-        gai_family = 0
+    if socket.has_ipv6 and not disable_ipv6:
         family = socket.AF_INET6
     else:
-        gai_family = family = socket.AF_INET
+        family = socket.AF_INET
 
-    sockaddr = ('', port)
-    if bindaddr is not None:
-        addrs = socket.getaddrinfo(bindaddr, port, gai_family)
-        family, socktype, proto, canonname, sockaddr = addrs[0]
+    sock = create_socket(family, bindaddr, port)
 
-    try:
-        sock = socket.socket(family, socket.SOCK_STREAM)
-    except socket.error as e:
-        # "An address incompatible with the requested protocol was used"
-        # - means IPv6 is not supported
-        if e.args[0] == 10047:
-            family = socket.AF_INET
-            sock = socket.socket(family, socket.SOCK_STREAM)
-        else:
-            raise
-
-    if family == socket.AF_INET6 and socket.has_ipv6:
-        # Windows needs this socket option set to listen on both IPV4 and
-        # IPV6. However, IPPROTO_IPV6 is missing from the socket module on Windows.
-        # Since the value is likely to never change, just include it here.
-        IPPROTO_IPV6 = 41
-        sock.setsockopt(IPPROTO_IPV6, socket.IPV6_V6ONLY, 0)
- 
-    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-    
-    _core._mark_non_inheritable(sock.fileno())
-
-    sock.bind(sockaddr)
     sock.listen(1)
 
     _server_socket = sock
